#ifndef DUNE_MICROSTRUCTURE_VOIGTHELPER_HH
#define DUNE_MICROSTRUCTURE_VOIGTHELPER_HH

/** \file
 * \brief Various helper functions related to Voigt notation
 */

#include <dune/common/fvector.hh>
#include <dune/common/fmatrix.hh>


template<typename T, int dim>
using VoigtVector = Dune::FieldVector<T,dim*(dim+1)/2>;

template<typename T, int dim>
using VoigtMatrix = Dune::FieldMatrix<T,dim*(dim+1)/2,dim*(dim+1)/2>;


template<typename T>
static VoigtVector<T,3> strainToVoigt(const Dune::FieldMatrix<T,3,3>& G)
{
  return {G[0][0], G[1][1], G[2][2], 2.0*G[1][2], 2.0*G[0][2], 2.0*G[0][1]};
}

template<typename T>
static Dune::FieldMatrix<T,3,3> voigtToStrain(const VoigtVector<T,3>& v)
{
  return {{v[0], v[5]/2.0, v[4]/2.0}, {v[5]/2.0, v[1], v[3]/2.0}, {v[4]/2.0, v[3]/2.0, v[2]}};
}

template<typename T>
static VoigtVector<T,3> stressToVoigt(const Dune::FieldMatrix<T,3,3>& G)
{
  return {G[0][0], G[1][1], G[2][2], G[1][2], G[0][2], G[0][1]};
}

template<typename T>
static Dune::FieldMatrix<T,3,3> voigtToStress(const VoigtVector<T,3>& v)
{
  return {{v[0], v[5], v[4]}, {v[5], v[1], v[3]}, {v[4], v[3], v[2]}};
}

#endif   // DUNE_MICROSTRUCTURE_VOIGTHELPER_HH
